#include "../../include/vmprofiler.hpp"

namespace vm
{
    namespace handler
    {
        namespace profile
        {
            vm::handler::profile_t mulq = {
                // MOV RAX, [RBP+0x8]
                // SUB RBP, 0x8
                // MUL RDX
                // MOV [RBP+0x8], RDX
                // MOV [RBP+0x10], RAX
                // PUSHFQ
                // POP [RBP]
                "MULQ",
                MULQ,
                NULL,
                { { // MOV RAX, [RBP+0x8]
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_RAX &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 1 ].mem.base == ZYDIS_REGISTER_RBP &&
                               instr.operands[ 1 ].mem.disp.value == 0x8;
                    },
                    // SUB RBP, 0x8
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_SUB &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_RBP &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_IMMEDIATE &&
                               instr.operands[ 1 ].imm.value.u == 0x8;
                    },
                    // MUL RDX
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MUL &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_RDX;
                    },
                    // MOV [RBP+0x8], RDX
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 0 ].mem.base == ZYDIS_REGISTER_RBP &&
                               instr.operands[ 0 ].mem.disp.value == 0x8 &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 1 ].reg.value == ZYDIS_REGISTER_RDX;
                    },
                    // MOV [RBP+0x10], RAX
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 0 ].mem.base == ZYDIS_REGISTER_RBP &&
                               instr.operands[ 0 ].mem.disp.value == 0x10 &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 1 ].reg.value == ZYDIS_REGISTER_RAX;
                    },
                    // PUSHFQ
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_PUSHFQ;
                    },
                    // POP [RBP]
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_POP &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 0 ].mem.base == ZYDIS_REGISTER_RBP;
                    } } } };
        }
    } // namespace handler
} // namespace vm