#include "../../include/vmprofiler.hpp"

namespace vm
{
    namespace handler
    {
        namespace profile
        {
            vm::handler::profile_t nandq = {
                // MOV RAX, [RBP]
                // MOV RDX, [RBP+8]
                // NOT RAX
                // NOT RDX
                // AND RAX, RDX
                // MOV [RBP], RAX
                // PUSHFQ
                // POP [RBP]
                "NANDQ",
                NANDQ,
                NULL,
                { { // MOV RAX, [RBP]
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_RAX &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 1 ].mem.base == ZYDIS_REGISTER_RBP;
                    },
                    // MOV RDX, [RBP+8]
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_RDX &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 1 ].mem.base == ZYDIS_REGISTER_RBP &&
                               instr.operands[ 1 ].mem.disp.value == 0x8;
                    },
                    // NOT RAX
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_NOT &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_RAX;
                    },
                    // NOT RDX
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_NOT &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_RDX;
                    },
                    // AND RAX, RDX
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_AND &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_RAX &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 1 ].reg.value == ZYDIS_REGISTER_RDX;
                    },
                    // MOV [RBP], RAX
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 0 ].mem.base == ZYDIS_REGISTER_RBP &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 1 ].reg.value == ZYDIS_REGISTER_RAX;
                    },
                    // PUSHFQ
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_PUSHFQ;
                    },
                    // POP [RBP]
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_POP &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 0 ].mem.base == ZYDIS_REGISTER_RBP;
                    } } } };

            vm::handler::profile_t nanddw = {
                // NOT QWORD PTR [RBP]
                // MOV EAX, [RBP]
                // SUB RBP, 0x4
                // AND [RBP+0x8], EAX
                // PUSHFQ
                // POP [RBP]
                "NANDDW",
                NANDDW,
                NULL,
                { { // NOT QWORD PTR [RBP]
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_NOT &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 0 ].size == 64 && instr.operands[ 0 ].mem.base == ZYDIS_REGISTER_RBP;
                    },
                    // MOV EAX, [RBP]
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_EAX &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 1 ].mem.base == ZYDIS_REGISTER_RBP;
                    },
                    // SUB RBP, 0x4
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_SUB &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_RBP &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_IMMEDIATE &&
                               instr.operands[ 1 ].imm.value.u == 0x4;
                    },
                    // AND [RBP+0x8], EAX
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_AND &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 0 ].mem.base == ZYDIS_REGISTER_RBP &&
                               instr.operands[ 0 ].mem.disp.value == 0x8 &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 1 ].reg.value == ZYDIS_REGISTER_EAX;
                    },
                    // PUSHFQ
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_PUSHFQ;
                    },
                    // POP [RBP]
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_POP &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 0 ].mem.base == ZYDIS_REGISTER_RBP;
                    } } } };

            vm::handler::profile_t nandw = {
                // NOT DWORD PTR [RBP]
                // MOV AX, [RBP]
                // SUB RBP, 0x6
                // AND [RBP+0x8], AX
                // PUSHFQ
                // POP [RBP]
                "NANDW",
                NANDW,
                NULL,
                { { // NOT DWORD PTR [RBP]
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_NOT &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 0 ].size == 32 && 
                               instr.operands[ 0 ].mem.base == ZYDIS_REGISTER_RBP;
                    },
                    // MOV AX, [RBP]
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_AX &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 1 ].mem.base == ZYDIS_REGISTER_RBP;
                    },
                    // SUB RBP, 0x6
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_SUB &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_RBP &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_IMMEDIATE &&
                               instr.operands[ 1 ].imm.value.u == 0x6;
                    },
                    // AND [RBP+0x8], AX
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_AND &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 0 ].mem.base == ZYDIS_REGISTER_RBP &&
                               instr.operands[ 0 ].mem.disp.value == 0x8 &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 1 ].reg.value == ZYDIS_REGISTER_AX;
                    },
                    // PUSHFQ
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_PUSHFQ;
                    },
                    // POP [RBP]
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_POP &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 0 ].mem.base == ZYDIS_REGISTER_RBP;
                    } } } };
        } // namespace profile
    }     // namespace handler
} // namespace vm