#include "../../include/vmprofiler.hpp"

namespace vm
{
    namespace handler
    {
        namespace profile
        {
            vm::handler::profile_t pushvsp = {
                // MOV RAX, RBP
                // SUB RBP, 8
                "PUSHVSP",
                PUSHVSP,
                NULL,
                { { // MOV RAX, RBP
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_RAX &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 1 ].reg.value == ZYDIS_REGISTER_RBP;
                    },
                    // SUB RBP, 8
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_SUB &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_RBP &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_IMMEDIATE &&
                               instr.operands[ 1 ].imm.value.u == 0x8;
                    },
                    // MOV [RBP], RAX
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 0 ].mem.base == ZYDIS_REGISTER_RBP &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 1 ].reg.value == ZYDIS_REGISTER_RAX;
                    } } } };
        }
    } // namespace handler
} // namespace vm