#include "../../include/vmprofiler.hpp"

namespace vm
{
    namespace handler
    {
        namespace profile
        {
            vm::handler::profile_t shrq = {
                // MOV RAX, [RBP]
                // MOV CL, [RBP+0x8]
                // SUB RBP, 0x6
                // SHR RAX, CL
                // MOV [RBP+0x8], RAX
                // PUSHFQ
                // POP [RBP]
                "SHRQ",
                SHRQ,
                NULL,
                { { // MOV RAX, [RBP]
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_RAX &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 1 ].mem.base == ZYDIS_REGISTER_RBP;
                    },
                    // MOV CL, [RBP+0x8]
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_CL &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 1 ].mem.base == ZYDIS_REGISTER_RBP &&
                               instr.operands[ 1 ].mem.disp.value == 0x8;
                    },
                    // SUB RBP, 0x6
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_SUB &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_RBP &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_IMMEDIATE &&
                               instr.operands[ 1 ].imm.value.u == 0x6;
                    },
                    // SHR RAX, CL
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_SHR &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_RAX &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 1 ].reg.value == ZYDIS_REGISTER_CL;
                    },
                    // MOV [RBP+0x8], RAX
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 0 ].mem.base == ZYDIS_REGISTER_RBP &&
                               instr.operands[ 0 ].mem.disp.value == 0x8 &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 1 ].reg.value == ZYDIS_REGISTER_RAX;
                    },
                    // PUSHFQ
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_PUSHFQ;
                    },
                    // POP [RBP]
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_POP &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 0 ].mem.base == ZYDIS_REGISTER_RBP;
                    } } } };
            
            vm::handler::profile_t shrw = {
                // MOV AX, [RBP]
                // MOV CL, [RBP+0x2]
                // SUB RBP, 0x6
                // SHR AX, CL
                // MOV [RBP+0x8], AX
                // PUSHFQ
                // POP [RBP]
                "SHRW",
                SHRW,
                NULL,
                { { // MOV AX, [RBP]
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_AX &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 1 ].mem.base == ZYDIS_REGISTER_RBP;
                    },
                    // MOV CL, [RBP+0x2]
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_CL &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 1 ].mem.base == ZYDIS_REGISTER_RBP &&
                               instr.operands[ 1 ].mem.disp.value == 0x2;
                    },
                    // SUB RBP, 0x6
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_SUB &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_RBP &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_IMMEDIATE &&
                               instr.operands[ 1 ].imm.value.u == 0x6;
                    },
                    // SHR AX, CL
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_SHR &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_AX &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 1 ].reg.value == ZYDIS_REGISTER_CL;
                    },
                    // MOV [RBP+0x8], AX
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 0 ].mem.base == ZYDIS_REGISTER_RBP &&
                               instr.operands[ 0 ].mem.disp.value == 0x8 &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 1 ].reg.value == ZYDIS_REGISTER_AX;
                    },
                    // PUSHFQ
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_PUSHFQ;
                    },
                    // POP [RBP]
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_POP &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 0 ].mem.base == ZYDIS_REGISTER_RBP;
                    } } } };
        }
    } // namespace handler
} // namespace vm