#include "../../include/vmprofiler.hpp"

namespace vm
{
    namespace handler
    {
        namespace profile
        {
            vm::handler::profile_t writeq = {
                // MOV RAX, [RBP]
                // MOV RDX, [RBP+0x8]
                // ADD RBP, 0x10
                // MOV [RAX], RDX
                "WRITEQ",
                WRITEQ,
                NULL,
                { { // MOV RAX, [RBP]
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_RAX &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 1 ].mem.base == ZYDIS_REGISTER_RBP;
                    },
                    // MOV RDX, [RBP+0x8]
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_RDX &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 1 ].mem.base == ZYDIS_REGISTER_RBP && 
                               instr.operands[ 1 ].mem.disp.value == 0x8;
                    },
                    // ADD RBP, 0x10
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_ADD &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_RBP &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_IMMEDIATE &&
                               instr.operands[ 1 ].imm.value.u == 0x10;
                    },
                    // MOV [RAX], RDX
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 0 ].mem.base == ZYDIS_REGISTER_RAX &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 1 ].reg.value == ZYDIS_REGISTER_RDX;
                    } } } };

            vm::handler::profile_t writedw = {
                // MOV RAX, [RBP]
                // MOV EDX, [RBP+0x8]
                // ADD RBP, 0xC
                // MOV [RAX], EDX
                "WRITEDW",
                WRITEDW,
                NULL,
                { { // MOV RAX, [RBP]
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_RAX &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 1 ].mem.base == ZYDIS_REGISTER_RBP;
                    },
                    // MOV EDX, [RBP+0x8]
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_EDX &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 1 ].mem.base == ZYDIS_REGISTER_RBP &&
                               instr.operands[ 1 ].mem.disp.value == 0x8;
                    },
                    // ADD RBP, 0xC
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_ADD &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_RBP &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_IMMEDIATE &&
                               instr.operands[ 1 ].imm.value.u == 0xC;
                    },
                    // MOV [RAX], EDX
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 0 ].mem.base == ZYDIS_REGISTER_RAX &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 1 ].reg.value == ZYDIS_REGISTER_EDX;
                    } } } };

            vm::handler::profile_t writeb = {
                // MOV RAX, [RBP]
                // MOV DL, [RBP+0x8]
                // ADD RBP, 0xA
                // MOV [RAX], DL
                "WRITEB",
                WRITEB,
                NULL,
                { { // MOV RAX, [RBP]
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_RAX &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 1 ].mem.base == ZYDIS_REGISTER_RBP;
                    },
                    // MOV DL, [RBP+0x8]
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_DL &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 1 ].mem.base == ZYDIS_REGISTER_RBP &&
                               instr.operands[ 1 ].mem.disp.value == 0x8;
                    },
                    // ADD RBP, 0xA
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_ADD &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 0 ].reg.value == ZYDIS_REGISTER_RBP &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_IMMEDIATE &&
                               instr.operands[ 1 ].imm.value.u == 0xA;
                    },
                    // MOV [RAX], DL
                    []( const zydis_decoded_instr_t &instr ) -> bool {
                        return instr.mnemonic == ZYDIS_MNEMONIC_MOV &&
                               instr.operands[ 0 ].type == ZYDIS_OPERAND_TYPE_MEMORY &&
                               instr.operands[ 0 ].mem.base == ZYDIS_REGISTER_RAX &&
                               instr.operands[ 1 ].type == ZYDIS_OPERAND_TYPE_REGISTER &&
                               instr.operands[ 1 ].reg.value == ZYDIS_REGISTER_DL;
                    } } } };
        } // namespace profile
    }     // namespace handler
} // namespace vm